#include "stdafx.h"
#include "Resource.h"
#include "TrayIconHandler.h"
#include "trayicon.h"
#include "CommandHelper.h"
#include "UpdaterApplication.h"
#include "UpdaterModel.h"
#include "UpdaterDocument.h"
#include "UpdaterView.h"

extern HMODULE _hModule;

TrayIconHandler *TrayIconHandler::m_TrayIconHandler = 0;
LPTOP_LEVEL_EXCEPTION_FILTER TrayIconHandler::exception_filter = 0;


LONG TrayIconfilter(struct _EXCEPTION_POINTERS* ExceptionInfo)
{
	TrayIconHandler::CreateInstacne()->_Hide();
	return TrayIconHandler::exception_filter(ExceptionInfo);
}


TrayIconHandler *TrayIconHandler::CreateInstacne()
{
	if( NULL == m_TrayIconHandler )
	{
		m_TrayIconHandler = new TrayIconHandler;
		if( m_TrayIconHandler )
		{
			m_TrayIconHandler->Create();
			exception_filter = SetUnhandledExceptionFilter( (LPTOP_LEVEL_EXCEPTION_FILTER)TrayIconfilter );
		}
	}

	return m_TrayIconHandler;
}

void TrayIconHandler::DestroyInstance()
{
	if( m_TrayIconHandler )
	{
		delete m_TrayIconHandler;
		m_TrayIconHandler = NULL;
	}

}

TrayIconHandler::TrayIconHandler() : m_TrayIcon(NULL)
{
	m_configModel = NULL;
}

TrayIconHandler::~TrayIconHandler()
{

}

void TrayIconHandler::SetUpdaterConfigModel( tipupdater::UpdaterConfigModel *configModel )
{
	m_configModel = configModel;
}

void TrayIconHandler::ShowTooltip(const TCHAR *text)
{
	if( m_TrayIcon )
	{
		m_TrayIcon->PopupBalloon(text);
	}
}

void TrayIconHandler::SetUpdatingToolTip()
{
	if( m_TrayIcon )
	{
		CString text = Application::LoadString(_T("IDS_TOOLTIP_TRAYICON_UPDATE") );
		m_TrayIcon->SetTooltipText(text);
	}
}

void TrayIconHandler::SetEmptyToolTip()
{
	if( m_TrayIcon )
	{
		CString text = Application::LoadString(_T("IDS_PROGRAM_NAME"));
		m_TrayIcon->SetTooltipText(text);
	}
}


BOOL TrayIconHandler::IsShow()
{
	return ( NULL != m_TrayIcon );
}

BOOL TrayIconHandler::Restart()
{
	if( m_TrayIcon )
	{
		m_TrayIcon->RemoveIcon();
		delete m_TrayIcon;
	}

	return _Show();
}

BOOL TrayIconHandler::TryShow()
{
	if( IsShow() )
		return TRUE;

	return _Show();
}

void TrayIconHandler::TryHide()
{
	if( FALSE == IsShow() )
		return;

	Application* application = Application::get_instance();
	tipupdater::UpdaterModel* updaterModel = dynamic_cast<tipupdater::UpdaterModel*>(application->GetModel());
	tipupdater::UpdaterDocument* document = updaterModel->GetDocument();
	tipupdater::UpdaterView* view = dynamic_cast<tipupdater::UpdaterView*>(application->GetView());
	
	if( view->GetMainWindow() != NULL )
	{
		return;
	}

	if( document->GetUpdaterConfigModel()->IsUseTrayIcon() != TRUE )
	{
		_Hide();
	}
}

BOOL TrayIconHandler::_Show()
{
	m_TrayIcon = new CTrayIcon;
	if( NULL == m_TrayIcon )
		return FALSE;

	HICON hIcon = LoadIcon(_hModule, MAKEINTRESOURCE(IDI_TIPUPDATER));
	CString text = Application::LoadString(_T("IDS_PROGRAM_NAME"));
	m_TrayIcon->Create( this->m_hWnd, WM_TRAYICON_CALLBACK_MESSAGE, text, hIcon, ID_TRAY_ICON );
	return TRUE;
}

void TrayIconHandler::_Hide()
{
	if( m_TrayIcon )
	{
		m_TrayIcon->RemoveIcon();
		delete m_TrayIcon;
		m_TrayIcon = NULL;
	}
}

LRESULT TrayIconHandler::OnQuit(WORD w1, WORD w2, HWND hWnd, BOOL& bHandled)
{
	return CommandHelper::ExecuteCommand(_T("QuitCommand"));
}

LRESULT TrayIconHandler::OnShowConfig(WORD w1, WORD w2, HWND hWnd, BOOL& bHandled)
{
	return CommandHelper::ExecuteCommand(_T("ShowUpdaterConfigDialogCommand"));
}

LRESULT TrayIconHandler::OnViewOpen(WORD w1, WORD w2, HWND hWnd, BOOL& bHandled)
{
	return CommandHelper::ExecuteCommand(_T("ViewOpenCommand"));
}

LRESULT TrayIconHandler::OnHide(WORD w1, WORD w2, HWND hWnd, BOOL& bHandled)
{
	Application* application = Application::get_instance();
	tipupdater::UpdaterModel* updaterModel = dynamic_cast<tipupdater::UpdaterModel*>(application->GetModel());
	tipupdater::UpdaterDocument* document = updaterModel->GetDocument();

	BOOL bFlag = document->GetUpdaterConfigModel()->IsUseTrayIcon();
	bFlag = !bFlag;

	document->GetUpdaterConfigModel()->SetUseTrayIcon(bFlag);
	document->SaveConfigModel();
	TryHide();

	return 1;
}

BOOL TrayIconHandler::IsUpdating()
{
	Application* application = Application::get_instance();
	tipupdater::UpdaterModel* updaterModel = dynamic_cast<tipupdater::UpdaterModel*>(application->GetModel());
	tipupdater::UpdaterDocument* document = updaterModel->GetDocument();

	return document->IsUpdating();
}

LRESULT TrayIconHandler::OnTrayIconCallback(UINT nMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	// context menu example
	if( LOWORD(lParam) == WM_RBUTTONUP )
	{
		if( IsUpdating() == TRUE )
			return 1;

		CMenu menuTemp;
		CMenuHandle contextMenu;
		CPoint pt;
		GetCursorPos(&pt);
		if( menuTemp.LoadMenu(IDR_TRAYICON_POPUP) )
		{
			contextMenu = menuTemp.GetSubMenu(0);

			if( m_configModel )
			{
				if( m_configModel->IsUseTrayIcon() == TRUE )
				{
					contextMenu.CheckMenuItem(ID_HIDE_TRAYICON, MF_UNCHECKED);
				}
				else
				{
					contextMenu.CheckMenuItem(ID_HIDE_TRAYICON, MF_CHECKED );
				}
			}
			SetForegroundWindow(this->m_hWnd);
			contextMenu.TrackPopupMenu(TPM_RIGHTALIGN | TPM_LEFTBUTTON, pt.x, pt.y, this->m_hWnd, NULL);
			PostMessage(WM_NULL);
		}
	}	
	return 1;
}